@@ -109,16 +109,14 @@ static GstFlowReturn gst_mad_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn gst_mad_change_state (GstElement * element,
     GstStateChange transition);
 
+#ifndef GST_DISABLE_INDEX
 static void gst_mad_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mad_get_index (GstElement * element);
+#endif
 
-static void
-_do_init (GType type)
-{
-  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
-}
+static GstTagList *gst_mad_id3_to_tag_list (const struct id3_tag *tag);
 
-GST_BOILERPLATE_FULL (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT, _do_init);
+GST_BOILERPLATE (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT);
 
 /*
 #define GST_TYPE_MAD_LAYER (gst_mad_layer_get_type())
@@ -192,6 +190,7 @@ gst_mad_base_init (gpointer g_class)
       gst_static_pad_template_get (&mad_src_template_factory));
   gst_element_class_set_details (element_class, &gst_mad_details);
 }
+
 static void
 gst_mad_class_init (GstMadClass * klass)
 {
@@ -208,8 +207,10 @@ gst_mad_class_init (GstMadClass * klass)
   gobject_class->dispose = gst_mad_dispose;
 
   gstelement_class->change_state = gst_mad_change_state;
+#ifndef GST_DISABLE_INDEX
   gstelement_class->set_index = gst_mad_set_index;
   gstelement_class->get_index = gst_mad_get_index;
+#endif
 
   /* init properties */
   /* currently, string representations are used, we might want to change that */
@@ -293,7 +294,9 @@ gst_mad_dispose (GObject * object)
 {
   GstMad *mad = GST_MAD (object);
 
+#ifndef GST_DISABLE_INDEX
   gst_mad_set_index (GST_ELEMENT (object), NULL);
+#endif
 
   g_free (mad->tempbuffer);
   mad->tempbuffer = NULL;
@@ -305,6 +308,7 @@ gst_mad_dispose (GObject * object)
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+#ifndef GST_DISABLE_INDEX
 static void
 gst_mad_set_index (GstElement * element, GstIndex * index)
 {
@@ -323,6 +327,7 @@ gst_mad_get_index (GstElement * element)
 
   return mad->index;
 }
+#endif
 
 static gboolean
 gst_mad_convert_sink (GstPad * pad, GstFormat src_format, gint64 src_value,
@@ -606,6 +611,7 @@ error:
   return FALSE;
 }
 
+#ifndef GST_DISABLE_INDEX
 static gboolean
 index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
 {
@@ -679,6 +685,7 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
 
   return FALSE;
 }
+#endif
 
 static gboolean
 normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
@@ -720,9 +727,6 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   /* shave off the flush flag, we'll need it later */
   flush = ((flags & GST_SEEK_FLAG_FLUSH) != 0);
 
-  /* assume the worst */
-  res = FALSE;
-
   conv = GST_FORMAT_BYTES;
   if (!gst_mad_convert_sink (pad, GST_FORMAT_TIME, time_cur, &conv, &bytes_cur))
     goto convert_error;
@@ -800,23 +804,24 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-      /* the all-formats seek logic */
     case GST_EVENT_SEEK:
+      /* the all-formats seek logic, ref the event, we need it later */
       gst_event_ref (event);
-      if (!(res = gst_pad_event_default (pad, event))) {
+      if (!(res = gst_pad_push_event (mad->sinkpad, event))) {
+#ifndef GST_DISABLE_INDEX
         if (mad->index)
           res = index_seek (mad, pad, event);
         else
+#endif
           res = normal_seek (mad, pad, event);
       }
+      gst_event_unref (event);
       break;
-
     default:
-      res = FALSE;
+      res = gst_pad_push_event (mad->sinkpad, event);
       break;
   }
 
-  gst_event_unref (event);
   return res;
 }
 
@@ -864,6 +869,7 @@ gst_mad_set_property (GObject * object, guint prop_id,
       break;
   }
 }
+
 static void
 gst_mad_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
@@ -888,7 +894,6 @@ gst_mad_get_property (GObject * object, guint prop_id,
 static void
 gst_mad_update_info (GstMad * mad)
 {
-  gint abr = mad->vbr_average;
   struct mad_header *header = &mad->frame.header;
   gboolean changed = FALSE;
 
@@ -904,7 +909,6 @@ G_STMT_START{                                                   \
   if (mad->new_header) {
     mad->framecount = 1;
     mad->vbr_rate = header->bitrate;
-    abr = 0;
   } else {
     mad->framecount++;
     mad->vbr_rate += header->bitrate;
@@ -1085,7 +1089,7 @@ mpg123_parse_xing_header (struct mad_header *header,
   int i;
   guint8 *ptr = (guint8 *) buf;
   double frame_duration;
-  int xflags, xframes, xbytes, xvbr_scale;
+  int xflags, xframes, xbytes;
   int abr;
   guint8 xtoc[XING_TOC_LENGTH];
   int lsf_bit = !(header->flags & MAD_FLAG_LSF_EXT);
@@ -1155,12 +1159,10 @@ mpg123_parse_xing_header (struct mad_header *header,
       ptr += XING_TOC_LENGTH;
     }
 
-    xvbr_scale = -1;
     if (xflags & XING_VBR_SCALE_FLAG) {
       if (ptr >= (buf + bufsize - 4))
         return 0;
-      xvbr_scale = BE_32 (ptr);
-      lprintf ("xvbr_scale: %d\n", xvbr_scale);
+      lprintf ("xvbr_scale: %d\n", BE_32 (ptr));
     }
 
     /* 1 kbit = 1000 bits ! (and not 1024 bits) */
@@ -1275,6 +1277,7 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
   guint8 *data;
   glong size, tempsize;
   gboolean new_pts = FALSE;
+  gboolean discont;
   GstClockTime timestamp;
   GstFlowReturn result = GST_FLOW_OK;
 
@@ -1286,8 +1289,12 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     GST_DEBUG ("mad restarted");
   }
 
+  /* take discont flag */
+  discont = GST_BUFFER_IS_DISCONT (buffer);
+
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
-  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
+  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT " duration:%" GST_TIME_FORMAT,
+      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
 
   /* handle timestamps */
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
@@ -1321,6 +1328,10 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     gint tocopy;
     guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
+    if (mad->tempsize == 0 && discont) {
+      mad->discont = TRUE;
+      discont = FALSE;
+    }
     tocopy =
         MIN (MAD_BUFFER_MDLEN, MIN (size,
             MAD_BUFFER_MDLEN * 3 - mad->tempsize));
@@ -1387,6 +1398,9 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
             GST_LOG ("sync error, flushing unneeded data");
             goto next_no_samples;
           }
+        } else if (mad->stream.error == MAD_ERROR_BADDATAPTR) {
+          /* Flush data */
+          goto next_no_samples;
         }
         /* we are in an error state */
         mad->in_error = TRUE;
@@ -1544,12 +1558,14 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
             GST_SECOND, mad->rate) - time_offset;
       }
 
+#ifndef GST_DISABLE_INDEX
       if (mad->index) {
         guint64 x_bytes = mad->base_byte_offset + mad->bytes_consumed;
 
         gst_index_add_association (mad->index, mad->index_id, 0,
             GST_FORMAT_BYTES, x_bytes, GST_FORMAT_TIME, time_offset, NULL);
       }
+#endif
 
       if (mad->segment_start <= (time_offset ==
               GST_CLOCK_TIME_NONE ? 0 : time_offset)) {
@@ -1604,12 +1620,14 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
 
         outdata = (gint32 *) GST_BUFFER_DATA (outbuffer);
 
-        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (time_offset));
+        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT " dur: %"
+            GST_TIME_FORMAT, GST_TIME_ARGS (time_offset),
+            GST_TIME_ARGS (time_duration));
 
         GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
         GST_BUFFER_DURATION (outbuffer) = time_duration;
         GST_BUFFER_OFFSET (outbuffer) = mad->total_samples;
+        GST_BUFFER_OFFSET_END (outbuffer) = mad->total_samples + nsamples;
 
         /* output sample(s) in 16-bit signed native-endian PCM */
         if (mad->channels == 1) {
@@ -1633,6 +1651,13 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
               GST_BUFFER_OFFSET (outbuffer),
               GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
+
+          /* apply discont */
+          if (mad->discont) {
+            GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_DISCONT);
+            mad->discont = FALSE;
+          }
+
           mad->segment.last_stop = GST_BUFFER_TIMESTAMP (outbuffer);
           result = gst_pad_push (mad->srcpad, outbuffer);
           if (result != GST_FLOW_OK) {
@@ -1656,6 +1681,10 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
         mad->bytes_consumed = 0;
       }
       tempsize = 0;
+      if (discont) {
+        mad->discont = TRUE;
+        discont = FALSE;
+      }
 
       if (gst_mad_check_restart (mad)) {
         goto end;
@@ -1707,6 +1736,7 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
       mad_frame_init (&mad->frame);
       mad_synth_init (&mad->synth);
       mad->tempsize = 0;
+      mad->discont = TRUE;
       mad->total_samples = 0;
       mad->rate = 0;
       mad->channels = 0;
@@ -1756,3 +1786,194 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
   }
   return ret;
 }
+
+/* id3 tag helper (FIXME: why does mad parse id3 tags at all? It shouldn't) */
+static GstTagList *
+gst_mad_id3_to_tag_list (const struct id3_tag *tag)
+{
+  const struct id3_frame *frame;
+  const id3_ucs4_t *ucs4;
+  id3_utf8_t *utf8;
+  GstTagList *tag_list;
+  GType tag_type;
+  guint i = 0;
+
+  tag_list = gst_tag_list_new ();
+
+  while ((frame = id3_tag_findframe (tag, NULL, i++)) != NULL) {
+    const union id3_field *field;
+    unsigned int nstrings, j;
+    const gchar *tag_name;
+
+    /* find me the function to query the frame id */
+    gchar *id = g_strndup (frame->id, 5);
+
+    tag_name = gst_tag_from_id3_tag (id);
+    if (tag_name == NULL) {
+      g_free (id);
+      continue;
+    }
+
+    if (strcmp (id, "COMM") == 0) {
+      if (frame->nfields < 4)
+        continue;
+
+      ucs4 = id3_field_getfullstring (&frame->fields[3]);
+      g_assert (ucs4);
+
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
+        continue;
+
+      if (!g_utf8_validate ((char *) utf8, -1, NULL)) {
+        GST_ERROR ("converted string is not valid utf-8");
+        g_free (utf8);
+        continue;
+      }
+
+      gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+          GST_TAG_COMMENT, utf8, NULL);
+
+      g_free (utf8);
+      continue;
+    }
+
+    if (frame->nfields < 2)
+      continue;
+
+    field = &frame->fields[1];
+    nstrings = id3_field_getnstrings (field);
+
+    for (j = 0; j < nstrings; ++j) {
+      ucs4 = id3_field_getstrings (field, j);
+      g_assert (ucs4);
+
+      if (strcmp (id, ID3_FRAME_GENRE) == 0)
+        ucs4 = id3_genre_name (ucs4);
+
+      utf8 = id3_ucs4_utf8duplicate (ucs4);
+      if (utf8 == 0)
+        continue;
+
+      if (!g_utf8_validate ((char *) utf8, -1, NULL)) {
+        GST_ERROR ("converted string is not valid utf-8");
+        free (utf8);
+        continue;
+      }
+
+      tag_type = gst_tag_get_type (tag_name);
+
+      /* be sure to add non-string tags here */
+      switch (tag_type) {
+        case G_TYPE_UINT:
+        {
+          guint tmp;
+          gchar *check;
+
+          tmp = strtoul ((char *) utf8, &check, 10);
+
+          if (strcmp (tag_name, GST_TAG_DATE) == 0) {
+            GDate *d;
+
+            if (*check != '\0')
+              break;
+            if (tmp == 0)
+              break;
+            d = g_date_new_dmy (1, 1, tmp);
+            tmp = g_date_get_julian (d);
+            g_date_free (d);
+          } else if (strcmp (tag_name, GST_TAG_TRACK_NUMBER) == 0) {
+            if (*check == '/') {
+              guint total;
+
+              check++;
+              total = strtoul (check, &check, 10);
+              if (*check != '\0')
+                break;
+
+              gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                  GST_TAG_TRACK_COUNT, total, NULL);
+            }
+          } else if (strcmp (tag_name, GST_TAG_ALBUM_VOLUME_NUMBER) == 0) {
+            if (*check == '/') {
+              guint total;
+
+              check++;
+              total = strtoul (check, &check, 10);
+              if (*check != '\0')
+                break;
+
+              gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+                  GST_TAG_ALBUM_VOLUME_COUNT, total, NULL);
+            }
+          }
+
+          if (*check != '\0')
+            break;
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, tag_name, tmp,
+              NULL);
+          break;
+        }
+        case G_TYPE_UINT64:
+        {
+          guint64 tmp;
+
+          g_assert (strcmp (tag_name, GST_TAG_DURATION) == 0);
+          tmp = strtoul ((char *) utf8, NULL, 10);
+          if (tmp == 0) {
+            break;
+          }
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+              GST_TAG_DURATION, tmp * 1000 * 1000, NULL);
+          break;
+        }
+        case G_TYPE_STRING:{
+          gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
+              tag_name, (const gchar *) utf8, NULL);
+          break;
+        }
+          /* handles GST_TYPE_DATE and anything else */
+        default:{
+          GValue src = { 0, };
+          GValue dest = { 0, };
+
+          g_value_init (&src, G_TYPE_STRING);
+          g_value_set_string (&src, (const gchar *) utf8);
+
+          g_value_init (&dest, tag_type);
+          if (g_value_transform (&src, &dest)) {
+            gst_tag_list_add_values (tag_list, GST_TAG_MERGE_APPEND,
+                tag_name, &dest, NULL);
+          } else {
+            GST_WARNING ("Failed to transform tag from string to type '%s'",
+                g_type_name (tag_type));
+          }
+          g_value_unset (&src);
+          g_value_unset (&dest);
+          break;
+        }
+      }
+      free (utf8);
+    }
+    g_free (id);
+  }
+
+  return tag_list;
+}
+
+/* plugin initialisation */
+
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
+
+  return gst_element_register (plugin, "mad", GST_RANK_SECONDARY,
+      gst_mad_get_type ());
+}
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "mad",
+    "mp3 decoding based on the mad library",
+    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
