@@ -62,6 +62,7 @@ struct _GstMPEGAudioParse {
 
   GstSegment segment;
   GstClockTime next_ts;
+  gboolean discont;
 
   /* Offset as supplied by incoming buffers */
   gint64 cur_offset;
@@ -119,6 +120,11 @@ struct _GstMPEGAudioParse {
   GSList *pending_accurate_seeks;
   gboolean exact_position;
 
+  /* Track whether we're seekable. The seek table for accurate seeking is
+   * not maintained if we're not seekable */
+  gboolean seekable;
+
+
   /* pending segment */
   GstEvent *pending_segment;
   /* pending events */
