@@ -1347,6 +1347,8 @@ setup_mpeg2dec ()
   mpeg2dec = gst_check_setup_element ("mpeg2dec");
   mysrcpad = gst_check_setup_src_pad (mpeg2dec, &srctemplate, NULL);
   mysinkpad = gst_check_setup_sink_pad (mpeg2dec, &sinktemplate, NULL);
+  gst_pad_set_active (mysrcpad, TRUE);
+  gst_pad_set_active (mysinkpad, TRUE);
 
   return mpeg2dec;
 }
@@ -1357,6 +1359,8 @@ cleanup_mpeg2dec (GstElement * mpeg2dec)
   GST_DEBUG ("cleanup_mpeg2dec");
   gst_element_set_state (mpeg2dec, GST_STATE_NULL);
 
+  gst_pad_set_active (mysrcpad, FALSE);
+  gst_pad_set_active (mysinkpad, FALSE);
   gst_check_teardown_src_pad (mpeg2dec);
   gst_check_teardown_sink_pad (mpeg2dec);
   gst_check_teardown_element (mpeg2dec);
@@ -1399,12 +1403,13 @@ GST_START_TEST (test_decode_stream1)
       gst_caps_new_simple ("video/x-raw-yuv", "format", GST_TYPE_FOURCC,
       GST_STR_FOURCC ("I420"), "width", G_TYPE_INT, 176, "height", G_TYPE_INT,
       144, "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1, "framerate",
-      GST_TYPE_FRACTION, 25, 1, NULL);
+      GST_TYPE_FRACTION, 25, 1, "interlaced", G_TYPE_BOOLEAN, FALSE, NULL);
 
   for (i = 0; i < num_buffers; ++i) {
     outbuffer = GST_BUFFER (buffers->data);
     fail_if (outbuffer == NULL);
 
+    GST_LOG ("buffer caps %" GST_PTR_FORMAT, GST_BUFFER_CAPS (outbuffer));
     fail_unless (gst_caps_is_equal_fixed (GST_BUFFER_CAPS (outbuffer),
             out_caps), "Incorrect buffer caps");
 
@@ -1465,12 +1470,13 @@ GST_START_TEST (test_decode_stream2)
       gst_caps_new_simple ("video/x-raw-yuv", "format", GST_TYPE_FOURCC,
       GST_STR_FOURCC ("I420"), "width", G_TYPE_INT, 183, "height", G_TYPE_INT,
       217, "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1, "framerate",
-      GST_TYPE_FRACTION, 25, 1, NULL);
+      GST_TYPE_FRACTION, 25, 1, "interlaced", G_TYPE_BOOLEAN, FALSE, NULL);
 
   for (i = 0; i < num_buffers; ++i) {
     outbuffer = GST_BUFFER (buffers->data);
     fail_if (outbuffer == NULL);
 
+    GST_LOG ("buffer caps %" GST_PTR_FORMAT, GST_BUFFER_CAPS (outbuffer));
     fail_unless (gst_caps_is_equal_fixed (GST_BUFFER_CAPS (outbuffer),
             out_caps), "Incorrect buffer caps");
 
